@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016-2021  Hanna Knutsson (hanna.knutsson@protonmail.com)
+    Copyright (C) 2003-2007, 2008, 2016-2024  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@
 #include <unistd.h>
 #include <stdint.h>
 
-#define QALCULATE_MAJOR_VERSION (4)
-#define QALCULATE_MINOR_VERSION (6)
+#define QALCULATE_MAJOR_VERSION (5)
+#define QALCULATE_MINOR_VERSION (1)
 #define QALCULATE_MICRO_VERSION (1)
 
 static std::string empty_string;
@@ -301,6 +301,7 @@ typedef enum {
 	SORT_SCIENTIFIC				= 1 << 1
 } SortFlags;
 
+/// Special values for PrintOptions::base and ParseOptions::base
 #define BASE_ROMAN_NUMERALS	-1
 #define BASE_TIME		-2
 #define BASE_BINARY		2
@@ -315,6 +316,7 @@ typedef enum {
 #define BASE_LATITUDE_2		71
 #define BASE_LONGITUDE		72
 #define BASE_LONGITUDE_2	73
+/// Use Calculate::setCustomOutputBase() or Calculate::setCustomInputBase() to specify a number base which greater than 36, negative, or a non-integer
 #define BASE_CUSTOM		-3
 #define BASE_UNICODE		-4
 #define BASE_GOLDEN_RATIO	-5
@@ -332,6 +334,7 @@ typedef enum {
 
 #define BASE_IS_SEXAGESIMAL(x) ((x >= BASE_SEXAGESIMAL && x <= BASE_SEXAGESIMAL_3) || (x >= BASE_LATITUDE && x <= BASE_LONGITUDE_2))
 
+/// Primary values for PrintOptions::min_exp
 #define EXP_BASE_3		-3
 #define EXP_PRECISION		-1
 #define EXP_NONE		0
@@ -346,7 +349,17 @@ typedef enum {
 	/// Display as fraction (ex. 4/3)
 	FRACTION_FRACTIONAL,
 	/// Display as an integer and a fraction (ex. 3 + 1/2)
-	FRACTION_COMBINED
+	FRACTION_COMBINED,
+	/// Display as fraction with denominator specified using Calculator::setFixedDenominator(). Both rational and non-rational numbers are rounded to match the selected denominator.
+	FRACTION_FRACTIONAL_FIXED_DENOMINATOR,
+	/// Display as an integer and a fraction with denominator specified using Calculator::setFixedDenominator(). Both rational and non-rational numbers are rounded to match the selected denominator.
+	FRACTION_COMBINED_FIXED_DENOMINATOR,
+	/// Display numbers in decimal format multiplied by percent
+	FRACTION_PERCENT,
+	/// Display numbers in decimal format multiplied by permille
+	FRACTION_PERMILLE,
+	/// Display numbers in decimal format multiplied by permyriad
+	FRACTION_PERMYRIAD
 } NumberFractionFormat;
 
 /// Options for ordering the parts of a mathematical expression/result before display
@@ -386,7 +399,9 @@ typedef enum {
 	INTERVAL_DISPLAY_PLUSMINUS,
 	INTERVAL_DISPLAY_MIDPOINT,
 	INTERVAL_DISPLAY_LOWER,
-	INTERVAL_DISPLAY_UPPER
+	INTERVAL_DISPLAY_UPPER,
+	INTERVAL_DISPLAY_CONCISE,
+	INTERVAL_DISPLAY_RELATIVE
 } IntervalDisplay;
 
 typedef enum {
@@ -406,6 +421,34 @@ typedef enum {
 	TIME_ZONE_CUSTOM
 } TimeZone;
 
+typedef enum {
+	EXP_DEFAULT,
+	EXP_UPPERCASE_E,
+	EXP_LOWERCASE_E,
+	EXP_POWER_OF_10
+} ExpDisplay;
+
+typedef enum {
+	ROUNDING_HALF_AWAY_FROM_ZERO,
+	ROUNDING_HALF_TO_EVEN,
+	ROUNDING_HALF_TO_ODD,
+	ROUNDING_HALF_TOWARD_ZERO,
+	ROUNDING_HALF_UP,
+	ROUNDING_HALF_DOWN,
+	ROUNDING_HALF_RANDOM,
+	ROUNDING_TOWARD_ZERO,
+	ROUNDING_AWAY_FROM_ZERO,
+	ROUNDING_UP,
+	ROUNDING_DOWN
+} RoundingMode;
+
+enum {
+	UNICODE_SIGNS_OFF,
+	UNICODE_SIGNS_ON,
+	UNICODE_SIGNS_ONLY_UNIT_EXPONENTS,
+	UNICODE_SIGNS_WITHOUT_EXPONENTS
+};
+
 // temporary custom time zone value for truncation rounding in output
 #define TZ_TRUNCATE -21586
 // temporary custom time zone value for special duodecimal symbols in output
@@ -413,14 +456,15 @@ typedef enum {
 
 /// Options for formatting and display of mathematical structures/results.
 struct PrintOptions {
+	/// Determines the minimum exponent with scientific notation. Values < -1 restricts the exponent to multiples of -min_exp, e.g. 20 000 is output as 20e3 (engineering notation) if min_exp is -3. The special value EXP_PRECISION (-1) uses the current precision to determine the minimum exponent. Default: EXP_PRECISION
 	int min_exp;
-	/// Number base for displaying numbers. Default: 10
+	/// Number base for displaying numbers. Specify a value between 2 and 36 or use one of the special values defined in includes.h (BASE_*). Default: 10
 	int base;
 	/// How prefixes for numbers in non-decimal bases will be displayed. Default: BASE_DISPLAY_NONE
 	BaseDisplay base_display;
 	/// Use lower case for non-numeric characters for bases > 10. Default: false
 	bool lower_case_numbers;
-	/// Use lower case e for base-10 exponent (ex. 1.2e8 instead of 1.2E8). Default: false
+	/// Deprecated: use exp_mode instead
 	bool lower_case_e;
 	/// If rational numbers will be displayed with decimals, as a fraction, or something in between. Default: FRACTION_DECIMAL
 	NumberFractionFormat number_fraction_format;
@@ -453,7 +497,7 @@ struct PrintOptions {
 	/// If it is not necessary that the displayed expression can be parsed correctly. Default: false
 	bool allow_non_usable;
 	/// If unicode signs can be displayed. Default: false
-	bool use_unicode_signs;
+	int use_unicode_signs;
 	/// Sign used for display of multiplication. Default: MULTIPLICATION_SIGN_DOT
 	MultiplicationSign multiplication_sign;
 	/// Sign used for display of division. Default: DIVISION_SIGN_DIVISION_SLASH
@@ -472,7 +516,7 @@ struct PrintOptions {
 	bool use_min_decimals;
 	/// Enable use of max_decimals. False is equivalent to a negative max_decimals value. Default: true
 	bool use_max_decimals;
-	/// If true round halfway numbers to nearest even number, otherwise round upwards. Default: false
+	/// Deprecated: use rounding
 	bool round_halfway_to_even;
 	/// Multiply numerator and denominator to get integers (ex. (6x+y)/2z instead of (3x+0.5y)/z). Default: true
 	bool improve_division_multipliers;
@@ -520,8 +564,15 @@ struct PrintOptions {
 	bool twos_complement;
 	/// Negative hexadecimal numbers uses two's complement representation. All hexadecimal numbers starting with 8 or higher are negative. Default: false
 	bool hexadecimal_twos_complement;
-	/// Number of bits used for binary numbers. Set to 0 for automatic. Default: 0
+	/// Number of bits used for binary and hexadecimal numbers. Set to 0 for automatic. Default: 0
 	unsigned int binary_bits;
+	/// Specifies how numbers using scientific notation are displayed. Default: EXP_DEFAULT
+	ExpDisplay exp_display;
+	/// Show special duodecimal symbols in output, instead of A and B. Default: false
+	bool duodecimal_symbols;
+	/// Specifies rounding method used in output. Default: ROUNDING_HALF_AWAY_FROM_ZERO
+	RoundingMode rounding;
+
 	PrintOptions();
 	/// Returns the comma sign used (default sign or comma_sign)
 	const std::string &comma() const;
@@ -601,7 +652,8 @@ typedef enum {
 	ANGLE_UNIT_NONE,
 	ANGLE_UNIT_RADIANS,
 	ANGLE_UNIT_DEGREES,
-	ANGLE_UNIT_GRADIANS
+	ANGLE_UNIT_GRADIANS,
+	ANGLE_UNIT_CUSTOM,
 } AngleUnit;
 
 typedef enum {
@@ -646,7 +698,7 @@ struct ParseOptions {
 	bool units_enabled;
 	/// If Reverse Polish Notation syntax will be used. Default: false (deprecated, use parsing_mode = PARSING_MODE_RPN instead)
 	bool rpn;
-	/// Base of parsed numbers. Default: 10
+	/// Base of parsed numbers. Specify a value between 2 and 36 or use one of the special values defined in includes.h (BASE_*). Default: 10
 	int base;
 	/// When implicit multiplication is limited variables, functions and units must be separated by a space, operator or parenthesis ("xy" does not equal "x * y").  Default: false
 	/**
@@ -671,10 +723,12 @@ struct ParseOptions {
 	DataSet *default_dataset;
 	/// Parsing mode. Default: PARSING_MODE_ADAPTIVE
 	ParsingMode parsing_mode;
-	/// Negative binary numbers uses two's complement representation. All binary numbers starting with 1 are assumed to be negative. Default: false
+	/// Negative binary numbers uses two's complement representation. All binary numbers starting with 1 are assumed to be negative, unless binary_bits is set. Default: false
 	bool twos_complement;
-	/// Negative hexadecimal numbers uses two's complement representation. All hexadecimal numbers starting with 8 or higher are assumed to be negative. Default: false
+	/// Negative hexadecimal numbers uses two's complement representation. All hexadecimal numbers starting with 8 or higher are assumed to be negative, unless binary_bits is set. Default: false
 	bool hexadecimal_twos_complement;
+	/// Number of bits used for binary and hexadecimal numbers. Set to 0 for automatic. Default: 0
+	unsigned int binary_bits;
 
 	ParseOptions();
 
